@@ -28,7 +28,6 @@ import android.content.Context;
 import android.databinding.ViewDataBinding;
 import android.graphics.drawable.BitmapDrawable;
 import android.support.annotation.MainThread;
-import android.support.v7.util.DiffUtil;
 import android.support.v7.widget.RecyclerView;
 import android.text.TextUtils;
 import android.view.LayoutInflater;
@@ -39,37 +38,38 @@ import android.widget.Filterable;
 
 import org.videolan.medialibrary.media.DummyItem;
 import org.videolan.medialibrary.media.MediaLibraryItem;
+import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.BR;
 import org.videolan.vlc.R;
-import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.SortableAdapter;
 import org.videolan.vlc.databinding.AudioBrowserItemBinding;
 import org.videolan.vlc.databinding.AudioBrowserSeparatorBinding;
-import org.videolan.vlc.gui.BaseQueuedAdapter;
 import org.videolan.vlc.gui.helpers.AsyncImageLoader;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.FastScroller;
 import org.videolan.vlc.interfaces.IEventsHandler;
-import org.videolan.vlc.util.MediaItemDiffCallback;
 import org.videolan.vlc.util.MediaItemFilter;
+import org.videolan.vlc.util.MediaLibraryItemComparator;
 import org.videolan.vlc.util.Util;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
 
-public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends MediaLibraryItem>, AudioBrowserAdapter.ViewHolder> implements FastScroller.SeparatedAdapter, Filterable {
+public class AudioBrowserAdapter extends SortableAdapter<MediaLibraryItem, AudioBrowserAdapter.ViewHolder> implements FastScroller.SeparatedAdapter, Filterable {
 
     private static final String TAG = "VLC/AudioBrowserAdapter";
 
     private boolean mMakeSections = true;
 
-    private ArrayList<? extends MediaLibraryItem> mDataList = new ArrayList<>();
-    private ArrayList<? extends MediaLibraryItem> mOriginalDataSet;
+    private ArrayList<MediaLibraryItem> mOriginalDataSet;
     private ItemFilter mFilter = new ItemFilter();
     private Activity mContext;
     private IEventsHandler mIEventsHandler;
     private int mSelectionCount = 0;
     private int mType;
+    private int mParentType = 0;
     private BitmapDrawable mDefaultCover;
 
     public AudioBrowserAdapter(Activity context, int type, IEventsHandler eventsHandler, boolean sections) {
@@ -80,6 +80,19 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
         mDefaultCover = getIconDrawable();
     }
 
+    public int getAdapterType() {
+        return mType;
+    }
+
+    public void setParentAdapterType(int type) {
+        mParentType = type;
+    }
+
+    public int getParentAdapterType() {
+        return mParentType;
+    }
+
+
     @Override
     public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
         LayoutInflater inflater = (LayoutInflater) parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
@@ -94,11 +107,11 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
 
     @Override
     public void onBindViewHolder(ViewHolder holder, int position) {
-        if (position >= mDataList.size())
+        if (position >= mDataset.size())
             return;
-        holder.vdb.setVariable(BR.item, mDataList.get(position));
+        holder.vdb.setVariable(BR.item, mDataset.get(position));
         if (holder.getType() == MediaLibraryItem.TYPE_MEDIA) {
-            boolean isSelected = mDataList.get(position).hasStateFlags(MediaLibraryItem.FLAG_SELECTED);
+            boolean isSelected = mDataset.get(position).hasStateFlags(MediaLibraryItem.FLAG_SELECTED);
             ((MediaItemViewHolder)holder).setCoverlay(isSelected);
             ((MediaItemViewHolder)holder).setViewBackground(holder.itemView.hasFocus(), isSelected);
         }
@@ -125,24 +138,24 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
 
     @Override
     public int getItemCount() {
-        return mDataList.size();
+        return mDataset.size();
     }
 
     public MediaLibraryItem getItem(int position) {
-        return isPositionValid(position) ? mDataList.get(position) : null;
+        return isPositionValid(position) ? mDataset.get(position) : null;
     }
 
     private boolean isPositionValid(int position) {
-        return position >= 0 || position < mDataList.size();
+        return position >= 0 || position < mDataset.size();
     }
 
-    public ArrayList<? extends MediaLibraryItem> getAll() {
-        return mDataList;
+    public ArrayList<MediaLibraryItem> getAll() {
+        return mDataset;
     }
 
     ArrayList<MediaLibraryItem> getMediaItems() {
         ArrayList<MediaLibraryItem> list = new ArrayList<>();
-        for (MediaLibraryItem item : mDataList)
+        for (MediaLibraryItem item : mDataset)
             if (!(item.getItemType() == MediaLibraryItem.TYPE_DUMMY)) list.add(item);
         return list;
     }
@@ -150,17 +163,17 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
     int getListWithPosition(ArrayList<MediaLibraryItem> list, int position) {
         int offset = 0, count = getItemCount();
         for (int i = 0; i < count; ++i)
-            if (mDataList.get(i).getItemType() == MediaLibraryItem.TYPE_DUMMY) {
+            if (mDataset.get(i).getItemType() == MediaLibraryItem.TYPE_DUMMY) {
                 if (i < position)
                     ++offset;
             } else
-                list.add(mDataList.get(i));
+                list.add(mDataset.get(i));
         return position-offset;
     }
 
     @Override
     public long getItemId(int position) {
-        return isPositionValid(position) ? mDataList.get(position).getId() : -1;
+        return isPositionValid(position) ? mDataset.get(position).getId() : -1;
     }
 
     @Override
@@ -176,7 +189,7 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
     public String getSectionforPosition(int position) {
         if (mMakeSections)
             for (int i = position; i >= 0; --i)
-                if (mDataList.get(i).getItemType() == MediaLibraryItem.TYPE_DUMMY) return mDataList.get(i).getTitle();
+                if (mDataset.get(i).getItemType() == MediaLibraryItem.TYPE_DUMMY) return mDataset.get(i).getTitle();
         return "";
     }
 
@@ -186,21 +199,21 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
     }
 
     public void clear() {
-        mDataList.clear();
+        mDataset.clear();
         mOriginalDataSet = null;
     }
 
 
-    public void addAll(ArrayList<? extends MediaLibraryItem> items) {
+    public void addAll(ArrayList<MediaLibraryItem> items) {
         addAll(items, mMakeSections);
     }
 
 
-    public void addAll(ArrayList<? extends MediaLibraryItem> items, boolean generateSections) {
+    public void addAll(ArrayList<MediaLibraryItem> items, boolean generateSections) {
         if (mContext == null)
             return;
-        mDataList = generateSections ? generateList(items) : items;
-        for (MediaLibraryItem item : mDataList) {
+        mDataset = new ArrayList<>(items);
+        for (MediaLibraryItem item : mDataset) {
             if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
                 continue;
             if (item.getTitle().isEmpty()) {
@@ -214,62 +227,137 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
                     if (TextUtils.isEmpty(item.getDescription()))
                         item.setDescription(mContext.getString(R.string.unknown_artist));
                 }
-            } else if (generateSections)
-                break;
+            }
         }
     }
 
+    public ArrayList<MediaLibraryItem> removeSections(ArrayList<MediaLibraryItem> items) {
+        ArrayList<MediaLibraryItem> newList = new ArrayList<>();
+        for (MediaLibraryItem item : items)
+            if (item.getItemType() != MediaLibraryItem.TYPE_DUMMY)
+                newList.add(item);
+        return newList;
+    }
 
-    private ArrayList<? extends MediaLibraryItem> generateList(ArrayList<? extends MediaLibraryItem> items) {
+    private ArrayList<MediaLibraryItem> generateSections(ArrayList<? extends MediaLibraryItem> items, int sortby) {
         ArrayList<MediaLibraryItem> datalist = new ArrayList<>();
-        boolean isLetter, emptyTitle;
-        String firstLetter = null, currentLetter = null;
-        for (MediaLibraryItem item : items) {
-            if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
-                continue;
-            String title = item.getTitle();
-            emptyTitle = title.isEmpty();
-            isLetter = !emptyTitle && Character.isLetter(title.charAt(0));
-            if (isLetter)
-                firstLetter = title.substring(0, 1).toUpperCase();
-            if (currentLetter == null) {
-                currentLetter = isLetter ? firstLetter : "#";
-                DummyItem sep = new DummyItem(currentLetter);
-                datalist.add(sep);
-            }
-            //Add a new separator
-            if (isLetter && !TextUtils.equals(currentLetter, firstLetter)) {
-                currentLetter = firstLetter;
-                DummyItem sep = new DummyItem(currentLetter);
-                datalist.add(sep);
-            }
-            datalist.add(item);
+        switch(sortby) {
+            case MediaLibraryItemComparator.SORT_BY_TITLE:
+                String currentLetter = null;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    String title = item.getTitle();
+                    String letter = (title.isEmpty() || !Character.isLetter(title.charAt(0))) ? "#" : title.substring(0, 1).toUpperCase();
+                    if (currentLetter == null || !TextUtils.equals(currentLetter, letter)) {
+                        currentLetter = letter;
+                        DummyItem sep = new DummyItem(currentLetter);
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
+            case MediaLibraryItemComparator.SORT_BY_ARTIST:
+                String currentArtist = null;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    String artist = ((MediaWrapper)item).getArtist();
+                    if (artist == null)
+                        artist = "";
+                    if (currentArtist == null || !TextUtils.equals(currentArtist, artist)) {
+                        currentArtist = artist;
+                        DummyItem sep = new DummyItem(TextUtils.isEmpty(currentArtist)
+                                ? mContext.getResources().getString(R.string.unknown_artist)
+                                : currentArtist);
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
+            case MediaLibraryItemComparator.SORT_BY_ALBUM:
+                String currentAlbum = null;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    String album = ((MediaWrapper)item).getAlbum();
+                    if (album == null)
+                        album = "";
+                    if (currentAlbum == null || !TextUtils.equals(currentAlbum, album)) {
+                        currentAlbum = album;
+                        DummyItem sep = new DummyItem(TextUtils.isEmpty(currentAlbum)
+                                ? mContext.getResources().getString(R.string.unknown_album)
+                                : currentAlbum);
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
+            case MediaLibraryItemComparator.SORT_BY_LENGTH :
+                String currentLengthCategory = null;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    int length = MediaLibraryItemComparator.getLength(item);
+                    String lengthCategory = MediaLibraryItemComparator.lengthToCategory(length);
+                    if (currentLengthCategory == null || !TextUtils.equals(currentLengthCategory, lengthCategory)) {
+                        currentLengthCategory = lengthCategory;
+                        DummyItem sep = new DummyItem(currentLengthCategory);
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
+            case MediaLibraryItemComparator.SORT_BY_DATE :
+                String currentYear = null;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    String year = MediaLibraryItemComparator.getYear(item);
+                    if (currentYear == null || !TextUtils.equals(currentYear, year)) {
+                        currentYear = year;
+                        DummyItem sep = new DummyItem(currentYear);
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
+            case MediaLibraryItemComparator.SORT_BY_NUMBER :
+                int currentNumber = 0;
+                for (MediaLibraryItem item : items) {
+                    if (item.getItemType() == MediaLibraryItem.TYPE_DUMMY)
+                        continue;
+                    int number = MediaLibraryItemComparator.getTracksCount(item);
+                    if (currentNumber != number) {
+                        currentNumber = number;
+                        DummyItem sep = new DummyItem(currentNumber == 0
+                                ? mContext.getResources().getString(R.string.unknown_number)
+                                : mContext.getResources().getQuantityString(R.plurals.songs_quantity, currentNumber, currentNumber));
+                        datalist.add(sep);
+                    }
+                    datalist.add(item);
+                }
+                break;
         }
         return datalist;
     }
 
     public void remove(final MediaLibraryItem item) {
-        final ArrayList<? extends MediaLibraryItem> referenceList = new ArrayList<>(peekLast());
+        final ArrayList<MediaLibraryItem> referenceList = new ArrayList<>(peekLast());
         if (referenceList.size() == 0) return;
-        final ArrayList<? extends MediaLibraryItem> dataList = new ArrayList<>(referenceList);
+        final ArrayList<MediaLibraryItem> dataList = new ArrayList<>(referenceList);
         dataList.remove(item);
         update(dataList);
     }
 
 
     public void addItem(final int position, final MediaLibraryItem item) {
-        final ArrayList<? extends MediaLibraryItem> referenceList = peekLast();
+        final ArrayList<MediaLibraryItem> referenceList = peekLast();
         final ArrayList<MediaLibraryItem> dataList = new ArrayList<>(referenceList);
         dataList.add(position,item);
         update(dataList);
     }
 
-
-    @Override
-    public ArrayList<? extends MediaLibraryItem> peekLast() {
-        return hasPendingUpdates() ? super.peekLast() : mDataList;
-    }
-
     public void restoreList() {
         if (mOriginalDataSet != null) {
             update(new ArrayList<>(mOriginalDataSet));
@@ -277,29 +365,31 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
         }
     }
 
-    protected void internalUpdate(final ArrayList<? extends MediaLibraryItem> items, final boolean detectMoves) {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                final ArrayList<? extends MediaLibraryItem> newList = (mOriginalDataSet == null && hasSections()) ? generateList(items) : items;
-                final DiffUtil.DiffResult result = DiffUtil.calculateDiff(new MediaItemDiffCallback(mDataList, newList), detectMoves);
-                VLCApplication.runOnMainThread(new Runnable() {
-                    @Override
-                    public void run() {
-                        addAll(newList, false);
-                        result.dispatchUpdatesTo(AudioBrowserAdapter.this);
-                        mIEventsHandler.onUpdateFinished(AudioBrowserAdapter.this);
-                        processQueue();
-                    }
-                });
+    @Override
+    protected void onUpdateFinished() {
+        super.onUpdateFinished();
+        mIEventsHandler.onUpdateFinished(AudioBrowserAdapter.this);
+    }
+
+    protected ArrayList<MediaLibraryItem> prepareList(ArrayList<MediaLibraryItem> items) {
+        if (mMakeSections) {
+            if (sMediaComparator.sortBy == MediaLibraryItemComparator.SORT_DEFAULT) {
+                return generateSections(items, getDefaultSort());
+            } else {
+                ArrayList<MediaLibraryItem> newList = removeSections(items);
+                Collections.sort(newList, sMediaComparator);
+                return generateSections(newList, sMediaComparator.sortBy);
             }
-        });
+        } else {
+            Collections.sort(items, sMediaComparator);
+            return items;
+        }
     }
 
     @MainThread
     public List<MediaLibraryItem> getSelection() {
         List<MediaLibraryItem> selection = new LinkedList<>();
-        for (MediaLibraryItem item : mDataList)
+        for (MediaLibraryItem item : mDataset)
             if (item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED))
                 selection.add(item);
         return selection;
@@ -360,20 +450,20 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
         public void onClick(View v) {
             if (mIEventsHandler != null) {
                 int position = getLayoutPosition();
-                mIEventsHandler.onClick(v, position, mDataList.get(position));
+                mIEventsHandler.onClick(v, position, mDataset.get(position));
             }
         }
 
         public void onMoreClick(View v) {
             if (mIEventsHandler != null) {
                 int position = getLayoutPosition();
-                mIEventsHandler.onCtxClick(v, position, mDataList.get(position));
+                mIEventsHandler.onCtxClick(v, position, mDataset.get(position));
             }
         }
 
         public boolean onLongClick(View view) {
             int position = getLayoutPosition();
-            return mIEventsHandler.onLongClick(view, position, mDataList.get(position));
+            return mIEventsHandler.onLongClick(view, position, mDataset.get(position));
         }
 
         private void setCoverlay(boolean selected) {
@@ -410,20 +500,64 @@ public class AudioBrowserAdapter extends BaseQueuedAdapter<ArrayList<? extends M
     private class ItemFilter extends MediaItemFilter {
 
         @Override
-        protected List<? extends MediaLibraryItem> initData() {
+        protected List<MediaLibraryItem> initData() {
             if (mOriginalDataSet == null) {
-                mOriginalDataSet = new ArrayList<>(mDataList);
+                mOriginalDataSet = new ArrayList<>(mDataset);
             }
             if (referenceList == null) {
-                referenceList = new ArrayList<>(mDataList);
+                referenceList = new ArrayList<>(mDataset);
             }
             return referenceList;
         }
 
         @Override
         protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
-            update((ArrayList<? extends MediaLibraryItem>) filterResults.values);
+            update((ArrayList<MediaLibraryItem>) filterResults.values);
         }
 
     }
+
+    public void sortBy(int sortby, int direction) {
+        boolean sort = isSortAllowed(sortby);
+        if (sort) {
+            super.sortBy(sortby, direction);
+        }
+    }
+
+    @Override
+    public int getDefaultSort() {
+        switch (mParentType) {
+            case MediaLibraryItem.TYPE_ARTIST:
+                return mType == MediaLibraryItem.TYPE_ALBUM ? MediaLibraryItemComparator.SORT_BY_DATE : MediaLibraryItemComparator.SORT_BY_ALBUM;
+            case MediaLibraryItem.TYPE_GENRE:
+                return mType == MediaLibraryItem.TYPE_ALBUM ? MediaLibraryItemComparator.SORT_BY_TITLE : MediaLibraryItemComparator.SORT_BY_ALBUM;
+            default:
+                return MediaLibraryItemComparator.SORT_BY_TITLE;
+        }
+    }
+
+    @Override
+    public int getDefaultDirection() {
+        return mParentType == MediaLibraryItem.TYPE_ARTIST ? -1 : 1;
+    }
+
+    @Override
+    protected boolean isSortAllowed(int sort) {
+        switch (sort) {
+            case MediaLibraryItemComparator.SORT_BY_TITLE:
+                return true;
+            case MediaLibraryItemComparator.SORT_BY_DATE:
+                return mType == MediaLibraryItem.TYPE_ALBUM;
+            case MediaLibraryItemComparator.SORT_BY_LENGTH:
+                return mType == MediaLibraryItem.TYPE_ALBUM || mType == MediaLibraryItem.TYPE_MEDIA;
+            case MediaLibraryItemComparator.SORT_BY_NUMBER:
+                return mType == MediaLibraryItem.TYPE_ALBUM || mType == MediaLibraryItem.TYPE_PLAYLIST;
+            case MediaLibraryItemComparator.SORT_BY_ALBUM:
+                return mParentType != 0 && mType == MediaLibraryItem.TYPE_MEDIA;
+            case MediaLibraryItemComparator.SORT_BY_ARTIST:
+                return mParentType == MediaLibraryItem.TYPE_GENRE && mType == MediaLibraryItem.TYPE_MEDIA;
+            default:
+                return false;
+        }
+    }
 }
