@@ -61,7 +61,6 @@ import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.FastScroller;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
-import org.videolan.vlc.interfaces.Filterable;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Util;
@@ -72,11 +71,9 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Random;
 
-public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, ViewPager.OnPageChangeListener, Medialibrary.ArtistsAddedCb, Medialibrary.ArtistsModifiedCb, Medialibrary.AlbumsAddedCb, Medialibrary.AlbumsModifiedCb, MediaAddedCb, MediaUpdatedCb, TabLayout.OnTabSelectedListener, Filterable {
+public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, ViewPager.OnPageChangeListener, Medialibrary.ArtistsAddedCb, Medialibrary.ArtistsModifiedCb, Medialibrary.AlbumsAddedCb, Medialibrary.AlbumsModifiedCb, MediaAddedCb, MediaUpdatedCb, TabLayout.OnTabSelectedListener {
     public final static String TAG = "VLC/AudioBrowserFragment";
 
-    private MainActivity mMainActivity;
-
     private AudioBrowserAdapter mArtistsAdapter;
     private AudioBrowserAdapter mAlbumsAdapter;
     private AudioBrowserAdapter mSongsAdapter;
@@ -87,9 +84,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     private TabLayout mTabLayout;
     private TextView mEmptyView;
     private ContextMenuRecyclerView[] mLists;
-    private AudioBrowserAdapter[] mAdapters;
     private FastScroller mFastScroller;
-    private View mSearchButtonView;
 
     public static final int REFRESH = 101;
     public static final int UPDATE_LIST = 102;
@@ -202,7 +197,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onAttach(Context context) {
         super.onAttach(context);
-        mMainActivity = (MainActivity) context;
+        mActivity = (MainActivity) context;
     }
 
     @Override
@@ -333,7 +328,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         }
 
         if (id == R.id.audio_view_add_playlist) {
-            UiTools.addToPlaylist(getActivity(), mediaItem.getTracks(mMediaLibrary));
+            UiTools.addToPlaylist(getActivity(), mediaItem.getTracks());
             return true;
         }
 
@@ -355,7 +350,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
             startPosition = 0;
             if (position >= adapter.getItemCount())
                 return false;
-            medias = mediaItem.getTracks(mMediaLibrary);
+            medias = mediaItem.getTracks();
         }
 
         if (mService != null) {
@@ -396,7 +391,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
 
     @Override
     public void onRefresh() {
-        mMainActivity.closeSearchView();
+        mActivity.closeSearchView();
         VLCApplication.getAppContext().startService(new Intent(MediaParsingService.ACTION_RELOAD, null, VLCApplication.getAppContext(), MediaParsingService.class));
     }
 
@@ -445,11 +440,6 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         getCurrentAdapter().restoreList();
     }
 
-    @Override
-    public void setSearchVisibility(boolean visible) {
-        mSearchButtonView.setVisibility(visible ? View.VISIBLE : View.GONE);
-    }
-
     private void updateEmptyView(int position) {
         mEmptyView.setVisibility(getCurrentAdapter().isEmpty() ? View.VISIBLE : View.GONE);
         mEmptyView.setText(position == MODE_PLAYLIST ? R.string.noplaylist : R.string.nomedia);
@@ -486,6 +476,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
 
     @Override
     public void onTabSelected(TabLayout.Tab tab) {
+        getActivity().supportInvalidateOptionsMenu();
         mFastScroller.setRecyclerView(mLists[tab.getPosition()]);
     }
 
@@ -493,7 +484,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     public void onTabUnselected(TabLayout.Tab tab) {
         stopActionMode();
         onDestroyActionMode((AudioBrowserAdapter) mLists[tab.getPosition()].getAdapter());
-        mMainActivity.closeSearchView();
+        mActivity.closeSearchView();
         mAdapters[tab.getPosition()].restoreList();
     }
 
@@ -506,7 +497,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                playlist.delete(mMediaLibrary);
+                playlist.delete();
                 mHandler.obtainMessage(UPDATE_LIST).sendToTarget();
             }
         });
@@ -567,7 +558,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onArtistsAdded() {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> artists = Util.arrayToArrayList(mMediaLibrary.getArtists());
+            final ArrayList<MediaLibraryItem> artists = Util.arrayToMediaArrayList(mMediaLibrary.getArtists());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -583,7 +574,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onArtistsModified() {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> artists = Util.arrayToArrayList(mMediaLibrary.getArtists());
+            final ArrayList<MediaLibraryItem> artists = Util.arrayToMediaArrayList(mMediaLibrary.getArtists());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -599,7 +590,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onAlbumsAdded() {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> albums = Util.arrayToArrayList(mMediaLibrary.getAlbums());
+            final ArrayList<MediaLibraryItem> albums = Util.arrayToMediaArrayList(mMediaLibrary.getAlbums());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -615,7 +606,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onAlbumsModified() {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> albums = Util.arrayToArrayList(mMediaLibrary.getAlbums());
+            final ArrayList<MediaLibraryItem> albums = Util.arrayToMediaArrayList(mMediaLibrary.getAlbums());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -631,7 +622,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onMediaAdded(MediaWrapper[] mediaList) {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> media = Util.arrayToArrayList(mMediaLibrary.getAudio());
+            final ArrayList<MediaLibraryItem> media = Util.arrayToMediaArrayList(mMediaLibrary.getAudio());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -647,7 +638,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     @Override
     public void onMediaUpdated(MediaWrapper[] mediaList) {
         VLCApplication.runBackground(new Runnable() {
-            final ArrayList<? extends MediaLibraryItem> media = Util.arrayToArrayList(mMediaLibrary.getAudio());
+            final ArrayList<MediaLibraryItem> media = Util.arrayToMediaArrayList(mMediaLibrary.getAudio());
             @Override
             public void run() {
                 VLCApplication.runOnMainThread(new Runnable() {
@@ -660,7 +651,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         });
     }
 
-    protected AudioBrowserAdapter getCurrentAdapter() {
+    public AudioBrowserAdapter getCurrentAdapter() {
         return (AudioBrowserAdapter) (getCurrentRV()).getAdapter();
     }
 
@@ -729,7 +720,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                final ArrayList<? extends MediaLibraryItem> artists = Util.arrayToArrayList(mMediaLibrary.getArtists());
+                final ArrayList<MediaLibraryItem> artists = Util.arrayToMediaArrayList(mMediaLibrary.getArtists());
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
@@ -744,7 +735,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                final ArrayList<? extends MediaLibraryItem> albums = Util.arrayToArrayList(mMediaLibrary.getAlbums());
+                final ArrayList<MediaLibraryItem> albums = Util.arrayToMediaArrayList(mMediaLibrary.getAlbums());
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
@@ -759,7 +750,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                final ArrayList<? extends MediaLibraryItem> media = Util.arrayToArrayList(mMediaLibrary.getAudio());
+                final ArrayList<MediaLibraryItem> media = Util.arrayToMediaArrayList(mMediaLibrary.getAudio());
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
@@ -774,7 +765,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                final ArrayList<? extends MediaLibraryItem> genres = Util.arrayToArrayList(mMediaLibrary.getGenres());
+                final ArrayList<MediaLibraryItem> genres = Util.arrayToMediaArrayList(mMediaLibrary.getGenres());
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
@@ -789,7 +780,7 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
         VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
-                final ArrayList<? extends MediaLibraryItem> playlists = Util.arrayToArrayList(mMediaLibrary.getPlaylists());
+                final ArrayList<MediaLibraryItem> playlists = Util.arrayToMediaArrayList(mMediaLibrary.getPlaylists());
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
@@ -837,4 +828,9 @@ public class AudioBrowserFragment extends BaseAudioBrowser implements SwipeRefre
     protected void onParsingServiceFinished() {
         mHandler.sendEmptyMessage(UPDATE_LIST);
     }
+
+    @Override
+    public int sortDirection(int sortby) {
+        return getCurrentAdapter().sortDirection(sortby);
+    }
 }
