@@ -23,6 +23,7 @@
 
 package org.videolan.vlc.gui.helpers;
 
+import android.annotation.TargetApi;
 import android.app.Activity;
 import android.content.Context;
 import android.content.SharedPreferences;
@@ -49,6 +50,8 @@ import android.text.Html;
 import android.text.TextUtils;
 import android.util.DisplayMetrics;
 import android.util.TypedValue;
+import android.view.Menu;
+import android.view.MenuItem;
 import android.view.View;
 import android.view.animation.Animation;
 import android.view.animation.AnimationSet;
@@ -58,12 +61,16 @@ import android.view.inputmethod.InputMethodManager;
 import android.widget.ImageView;
 import android.widget.TextView;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.medialibrary.media.MediaLibraryItem;
 import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.BuildConfig;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.audio.BaseAudioBrowser;
+import org.videolan.vlc.gui.browser.SortableFragment;
 import org.videolan.vlc.gui.dialogs.SavePlaylistDialog;
+import org.videolan.vlc.util.MediaLibraryItemComparator;
 
 import java.util.List;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -86,8 +93,12 @@ public class UiTools {
     }
 
     /** Print an on-screen message to alert the user */
+    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
     public static void snacker(@NonNull View view, @NonNull String message) {
-        Snackbar.make(view, message, Snackbar.LENGTH_SHORT).show();
+        Snackbar snack = Snackbar.make(view, message, Snackbar.LENGTH_SHORT);
+        if (AndroidUtil.isLolliPopOrLater)
+            snack.getView().setElevation(view.getResources().getDimensionPixelSize(R.dimen.audio_player_elevation));
+        snack.show();
     }
 
     /** Print an on-screen message to alert the user, with undo action */
@@ -96,8 +107,9 @@ public class UiTools {
     }
 
     /** Print an on-screen message to alert the user, with undo action */
+    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
     public static void snackerWithCancel(@NonNull View view, @NonNull String message, @Nullable final Runnable action, @Nullable final Runnable cancelAction) {
-        Snackbar.make(view, message, DELETE_DURATION)
+        Snackbar snack = Snackbar.make(view, message, DELETE_DURATION)
                 .setAction(android.R.string.cancel, new View.OnClickListener() {
                     @Override
                     public void onClick(View v) {
@@ -106,8 +118,10 @@ public class UiTools {
                         if (cancelAction != null)
                             cancelAction.run();
                     }
-                })
-                .show();
+                });
+        if (AndroidUtil.isLolliPopOrLater)
+            snack.getView().setElevation(view.getResources().getDimensionPixelSize(R.dimen.audio_player_elevation));
+        snack.show();
         if (action != null)
             sHandler.postDelayed(action, DELETE_DURATION);
     }
@@ -205,19 +219,19 @@ public class UiTools {
     }
 
     public static void fillAboutView(View v) {
-        TextView link = (TextView) v.findViewById(R.id.main_link);
+        final TextView link = v.findViewById(R.id.main_link);
         link.setText(Html.fromHtml(VLCApplication.getAppResources().getString(R.string.about_link)));
 
-        String revision = VLCApplication.getAppResources().getString(R.string.build_revision)+" VLC: "+VLCApplication.getAppResources().getString(R.string.build_vlc_revision);
-        String builddate = VLCApplication.getAppResources().getString(R.string.build_time);
-        String builder = VLCApplication.getAppResources().getString(R.string.build_host);
+        final String revision = VLCApplication.getAppResources().getString(R.string.build_revision)+" VLC: "+VLCApplication.getAppResources().getString(R.string.build_vlc_revision);
+        final String builddate = VLCApplication.getAppResources().getString(R.string.build_time);
+        final String builder = VLCApplication.getAppResources().getString(R.string.build_host);
 
-        TextView compiled = (TextView) v.findViewById(R.id.main_compiled);
+        final TextView compiled = v.findViewById(R.id.main_compiled);
         compiled.setText(builder + " (" + builddate + ")");
-        TextView textview_rev = (TextView) v.findViewById(R.id.main_revision);
+        final TextView textview_rev = v.findViewById(R.id.main_revision);
         textview_rev.setText(VLCApplication.getAppResources().getString(R.string.revision) + " " + revision + " (" + builddate + ") " + BuildConfig.FLAVOR_abi);
 
-        final ImageView logo = (ImageView) v.findViewById(R.id.logo);
+        final ImageView logo = v.findViewById(R.id.logo);
         logo.setOnClickListener(new View.OnClickListener() {
             @Override
             public void onClick(View v) {
@@ -316,4 +330,69 @@ public class UiTools {
 
 		return outBitmap;
 	}
+
+    public static void updateSortTitles(SortableFragment sortable, Menu menu) {
+        MenuItem item = menu.findItem(R.id.ml_menu_sortby_name);
+        if (item != null) {
+            if (sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_TITLE) == 1
+                    || (sortable.getSortBy() == MediaLibraryItemComparator.SORT_DEFAULT
+                    && sortable.getDefaultSort() == MediaLibraryItemComparator.SORT_BY_TITLE))
+                item.setTitle(R.string.sortby_name_desc);
+            else
+                item.setTitle(R.string.sortby_name);
+        }
+
+        if (sortable instanceof BaseAudioBrowser && sortable.sortDirection(MediaLibraryItemComparator.SORT_DEFAULT) == 1) {
+            int defaultSortby = ((BaseAudioBrowser)sortable).getCurrentAdapter().getDefaultSort();
+            int defaultDirection = ((BaseAudioBrowser)sortable).getCurrentAdapter().getDefaultDirection();
+            menu.findItem(R.id.ml_menu_sortby_length).setTitle(R.string.sortby_length);
+            menu.findItem(R.id.ml_menu_sortby_number).setTitle(R.string.sortby_number);
+            menu.findItem(R.id.ml_menu_sortby_artist_name).setTitle(R.string.sortby_artist_name);
+            menu.findItem(R.id.ml_menu_sortby_name).setTitle(defaultSortby == MediaLibraryItemComparator.SORT_BY_TITLE && defaultDirection == 1
+                    ? R.string.sortby_name_desc
+                    : R.string.sortby_name);
+            menu.findItem(R.id.ml_menu_sortby_date).setTitle(defaultSortby == MediaLibraryItemComparator.SORT_BY_DATE && defaultDirection == 1
+                    ? R.string.sortby_date_desc
+                    : R.string.sortby_date);
+            menu.findItem(R.id.ml_menu_sortby_album_name).setTitle(defaultSortby == MediaLibraryItemComparator.SORT_BY_ALBUM && defaultDirection == 1
+                    ? R.string.sortby_album_name_desc
+                    : R.string.sortby_album_name);
+            return;
+        }
+        item = menu.findItem(R.id.ml_menu_sortby_artist_name);
+        if (item != null) {
+            if (sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_ARTIST) == 1)
+                item.setTitle(R.string.sortby_artist_name_desc);
+            else
+                item.setTitle(R.string.sortby_artist_name);
+        }
+        item = menu.findItem(R.id.ml_menu_sortby_album_name);
+        if (item != null) {
+            if (sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_ALBUM) == 1)
+                item.setTitle(R.string.sortby_album_name_desc);
+            else
+                item.setTitle(R.string.sortby_album_name);
+        }
+        item = menu.findItem(R.id.ml_menu_sortby_length);
+        if (item != null) {
+            if (sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_LENGTH) == 1)
+                item.setTitle(R.string.sortby_length_desc);
+            else
+                item.setTitle(R.string.sortby_length);
+        }
+        item = menu.findItem(R.id.ml_menu_sortby_date);
+        if (item != null) {
+            if(sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_DATE) == 1)
+                item.setTitle(R.string.sortby_date_desc);
+            else
+                item.setTitle(R.string.sortby_date);
+        }
+        item = menu.findItem(R.id.ml_menu_sortby_number);
+        if (item != null) {
+            if (sortable.sortDirection(MediaLibraryItemComparator.SORT_BY_NUMBER) == 1)
+                item.setTitle(R.string.sortby_number_desc);
+            else
+                item.setTitle(R.string.sortby_number);
+        }
+    }
 }
