@@ -24,20 +24,14 @@
 package org.videolan.vlc.gui.preferences;
 
 import android.content.Intent;
-import android.content.SharedPreferences;
 import android.os.Bundle;
-import android.support.v7.preference.ListPreference;
 import android.support.v7.preference.Preference;
-import android.support.v7.preference.PreferenceManager;
 
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.SecondaryActivity;
-import org.videolan.vlc.util.AndroidDevices;
-import org.videolan.vlc.util.Util;
-import org.videolan.vlc.util.VLCInstance;
 
-public class PreferencesFragment extends BasePreferenceFragment implements SharedPreferences.OnSharedPreferenceChangeListener {
+public class PreferencesFragment extends BasePreferenceFragment {
 
     public final static String TAG = "VLC/PreferencesFragment";
 
@@ -54,35 +48,13 @@ public class PreferencesFragment extends BasePreferenceFragment implements Share
     }
 
     @Override
-    public void onCreate(Bundle savedInstanceState) {
-        super.onCreate(savedInstanceState);
-
-        // Screen orientation
-        ListPreference screenOrientationPref = (ListPreference) findPreference("screen_orientation");
-        screenOrientationPref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
-            @Override
-            public boolean onPreferenceChange(Preference preference, Object newValue) {
-                final SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(getActivity());
-                SharedPreferences.Editor editor = sharedPrefs.edit();
-                editor.putString("screen_orientation_value", (String) newValue);
-                Util.commitPreferences(editor);
-                return true;
-            }
-        });
-
-        /*** SharedPreferences Listener to apply changes ***/
-        SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(getActivity());
-        sharedPrefs.registerOnSharedPreferenceChangeListener(this);
+    public void onStart() {
+        super.onStart();
     }
 
     @Override
-    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
-        if(key.equalsIgnoreCase("hardware_acceleration")
-                || key.equalsIgnoreCase("subtitle_text_encoding")) {
-            VLCInstance.restart();
-            if (getActivity() != null )
-                ((PreferencesActivity)getActivity()).restartMediaPlayer();
-        }
+    public void onCreate(Bundle savedInstanceState) {
+        super.onCreate(savedInstanceState);
     }
 
     @Override
@@ -97,14 +69,17 @@ public class PreferencesFragment extends BasePreferenceFragment implements Share
             case "ui_category":
                 loadFragment(new PreferencesUi());
                 break;
-            case "perf_category":
-                loadFragment(new PreferencesPerformances());
+            case "video_category":
+                loadFragment(new PreferencesVideo());
+                break;
+            case "subtitles_category":
+                loadFragment(new PreferencesSubtitles());
+                break;
+            case "audio_category":
+                loadFragment(new PreferencesAudio());
                 break;
             case "adv_category":
-                loadFragment(new Advanced());
-                break;
-            case "dev_category":
-                loadFragment(new Developer());
+                loadFragment(new PreferencesAdvanced());
                 break;
             case PLAYBACK_HISTORY:
                 getActivity().setResult(PreferencesActivity.RESULT_RESTART);
