@@ -20,6 +20,8 @@
  *****************************************************************************/
 package org.videolan.vlc.gui.tv;
 
+import android.animation.ValueAnimator;
+import android.annotation.TargetApi;
 import android.app.Activity;
 import android.content.Context;
 import android.content.Intent;
@@ -46,6 +48,7 @@ import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
 import org.videolan.vlc.gui.tv.browser.VerticalGridActivity;
 import org.videolan.vlc.media.MediaUtils;
 
+import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Arrays;
 
@@ -53,6 +56,7 @@ import static org.videolan.vlc.gui.tv.browser.MusicFragment.AUDIO_CATEGORY;
 import static org.videolan.vlc.gui.tv.browser.MusicFragment.AUDIO_ITEM;
 import static org.videolan.vlc.gui.tv.browser.MusicFragment.CATEGORY_ALBUMS;
 
+@TargetApi(Build.VERSION_CODES.JELLY_BEAN)
 public class TvUtil {
 
     public static void applyOverscanMargin(Activity activity) {
@@ -126,7 +130,7 @@ public class TvUtil {
 
     public static void openAudioCategory(Activity context, MediaLibraryItem mediaLibraryItem) {
         if (mediaLibraryItem.getItemType() == MediaLibraryItem.TYPE_ALBUM) {
-            TvUtil.playAudioList(context, mediaLibraryItem.getTracks(VLCApplication.getMLInstance()), 0);
+            TvUtil.playAudioList(context, mediaLibraryItem.getTracks(), 0);
         } else {
             Intent intent = new Intent(context, VerticalGridActivity.class);
             intent.putExtra(AUDIO_ITEM, mediaLibraryItem);
@@ -169,4 +173,17 @@ public class TvUtil {
         bm.setColor(ContextCompat.getColor(VLCApplication.getAppContext(), R.color.tv_bg));
         bm.setDrawable(null);
     }
+
+    //See https://issuetracker.google.com/issues/37135111
+    public static void releaseBackgroundManager(BackgroundManager backgroundManager) {
+        Field field;
+        try {
+            field = backgroundManager.getClass().getDeclaredField("mAnimator");
+            field.setAccessible(true);
+            ValueAnimator valueAnimator = (ValueAnimator) field.get(backgroundManager);
+            if (valueAnimator != null && valueAnimator.isStarted())
+                valueAnimator.cancel();
+        } catch (Exception ignored) {}
+        backgroundManager.release();
+    }
 }
