@@ -1,7 +1,7 @@
 /*****************************************************************************
  * VideoListActivity.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2017 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,14 +61,13 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
-import org.videolan.vlc.gui.browser.MediaBrowserFragment;
+import org.videolan.vlc.gui.browser.SortableFragment;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.AutoFitRecyclerView;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.Filterable;
 import org.videolan.vlc.interfaces.IEventsHandler;
-import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.media.MediaGroup;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.util.FileUtils;
@@ -77,7 +76,7 @@ import org.videolan.vlc.util.VLCInstance;
 import java.util.ArrayList;
 import java.util.List;
 
-public class VideoGridFragment extends MediaBrowserFragment implements MediaUpdatedCb, ISortable, SwipeRefreshLayout.OnRefreshListener, MediaAddedCb, Filterable, IEventsHandler {
+public class VideoGridFragment extends SortableFragment<VideoListAdapter> implements MediaUpdatedCb, SwipeRefreshLayout.OnRefreshListener, MediaAddedCb, Filterable, IEventsHandler {
 
     public final static String TAG = "VLC/VideoListFragment";
 
@@ -89,7 +88,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     protected View mViewNomedia;
     protected String mGroup;
     private View mSearchButtonView;
-    private VideoListAdapter mVideoAdapter;
     private DividerItemDecoration mDividerItemDecoration;
 
     /* All subclasses of Fragment must include a public empty constructor. */
@@ -99,12 +97,34 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        mVideoAdapter = new VideoListAdapter(this);
+        mAdapter = new VideoListAdapter(this);
 
         if (savedInstanceState != null)
             setGroup(savedInstanceState.getString(KEY_GROUP));
     }
 
+
+    @Override
+    public void onPrepareOptionsMenu(Menu menu) {
+        super.onPrepareOptionsMenu(menu);
+        menu.findItem(R.id.ml_menu_sortby_artist_name).setVisible(false);
+        menu.findItem(R.id.ml_menu_sortby_album_name).setVisible(false);
+        menu.findItem(R.id.ml_menu_sortby_length).setVisible(true);
+        menu.findItem(R.id.ml_menu_sortby_date).setVisible(true);
+        menu.findItem(R.id.ml_menu_last_playlist).setVisible(true);
+    }
+
+    @Override
+    public boolean onOptionsItemSelected(MenuItem item) {
+        switch (item.getItemId()) {
+            case R.id.ml_menu_last_playlist:
+                getActivity().sendBroadcast(new Intent(PlaybackService.ACTION_REMOTE_LAST_VIDEO_PLAYLIST));
+                return true;
+            default:
+                return super.onOptionsItemSelected(item);
+        }
+    }
+
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState){
 
@@ -121,9 +141,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
         mSwipeRefreshLayout.setOnRefreshListener(this);
 
         mDividerItemDecoration = new DividerItemDecoration(v.getContext(), DividerItemDecoration.VERTICAL);
-        if (mVideoAdapter.isListMode())
+        if (mAdapter.isListMode())
             mGridView.addItemDecoration(mDividerItemDecoration);
-        mGridView.setAdapter(mVideoAdapter);
+        mGridView.setAdapter(mAdapter);
         return v;
     }
 
@@ -165,7 +185,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     @Override
     public void onDestroy() {
         super.onDestroy();
-        mVideoAdapter.clear();
+        mAdapter.clear();
     }
 
     protected void onMedialibraryReady() {
@@ -201,15 +221,15 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
         if (!listMode) {
             int thumbnailWidth = res.getDimensionPixelSize(R.dimen.grid_card_thumb_width);
             mGridView.setColumnWidth(mGridView.getPerfectColumnWidth(thumbnailWidth, res.getDimensionPixelSize(R.dimen.default_margin)));
-            mVideoAdapter.setGridCardWidth(mGridView.getColumnWidth());
+            mAdapter.setGridCardWidth(mGridView.getColumnWidth());
         }
         mGridView.setNumColumns(listMode ? 1 : -1);
-        if (mVideoAdapter.isListMode() != listMode) {
+        if (mAdapter.isListMode() != listMode) {
             if (listMode)
                 mGridView.addItemDecoration(mDividerItemDecoration);
             else
                 mGridView.removeItemDecoration(mDividerItemDecoration);
-            mVideoAdapter.setListMode(listMode);
+            mAdapter.setListMode(listMode);
         }
     }
 
@@ -230,9 +250,9 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     }
 
     protected boolean handleContextItemSelected(MenuItem menu, final int position) {
-        if (position >= mVideoAdapter.getItemCount())
+        if (position >= mAdapter.getItemCount())
             return false;
-        final MediaWrapper media = mVideoAdapter.getItem(position);
+        final MediaWrapper media = mAdapter.getItem(position);
         if (media == null)
             return false;
         switch (menu.getItemId()){
@@ -244,7 +264,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
                 return true;
             case R.id.video_list_play_all:
                 ArrayList<MediaWrapper> playList = new ArrayList<>();
-                MediaUtils.openList(getActivity(), playList, mVideoAdapter.getListWithPosition(playList, position));
+                MediaUtils.openList(getActivity(), playList, mAdapter.getListWithPosition(playList, position));
                 return true;
             case R.id.video_list_info:
                 showInfoDialog(media);
@@ -269,7 +289,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     }
 
     private void removeVideo(final MediaWrapper media) {
-        mVideoAdapter.remove(media);
+        mAdapter.remove(media);
         if (getView() != null)
             UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
                 @Override
@@ -279,7 +299,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
             }, new Runnable() {
                 @Override
                 public void run() {
-                    mVideoAdapter.add(media);
+                    mAdapter.add(media);
                 }
             });
     }
@@ -290,7 +310,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
             return;
         // Do not show the menu of media group.
         ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView.RecyclerContextMenuInfo)menuInfo;
-        MediaWrapper media = mVideoAdapter.getItem(info.position);
+        MediaWrapper media = mAdapter.getItem(info.position);
         if (media == null)
             return;
         MenuInflater inflater = getActivity().getMenuInflater();
@@ -327,7 +347,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     @Override
     public void onFabPlayClick(View view) {
         ArrayList<MediaWrapper> playList = new ArrayList<>();
-        MediaUtils.openList(getActivity(), playList, mVideoAdapter.getListWithPosition(playList, 0));
+        MediaUtils.openList(getActivity(), playList, mAdapter.getListWithPosition(playList, 0));
     }
 
     @Override
@@ -346,7 +366,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
                 mHandler.post(new Runnable() {
                     @Override
                     public void run() {
-                        mVideoAdapter.update(mw);
+                        mAdapter.update(mw);
                         updateEmptyView();
                     }
                 });
@@ -374,7 +394,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
                 VLCApplication.runOnMainThread(new Runnable() {
                     @Override
                     public void run() {
-                        mVideoAdapter.update(displayList, false);
+                        mAdapter.update(displayList);
                     }
                 });
                 mHandler.sendEmptyMessage(UNSET_REFRESHING);
@@ -383,17 +403,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     }
 
     void updateEmptyView() {
-        mViewNomedia.setVisibility(mVideoAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
-    }
-
-    @Override
-    public void sortBy(int sortby) {
-        mVideoAdapter.sortBy(sortby);
-    }
-
-    @Override
-    public int sortDirection(int sortby) {
-        return mVideoAdapter.sortDirection(sortby);
+        mViewNomedia.setVisibility(mAdapter.getItemCount() > 0 ? View.GONE : View.VISIBLE);
     }
 
     public void setGroup(String prefix) {
@@ -409,12 +419,12 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     public void display() {}
 
     public void clear(){
-        mVideoAdapter.clear();
+        mAdapter.clear();
     }
 
     @Override
     public void setFabPlayVisibility(boolean enable) {
-        super.setFabPlayVisibility(!mVideoAdapter.isEmpty() && enable);
+        super.setFabPlayVisibility(!mAdapter.isEmpty() && enable);
     }
 
     @Override
@@ -434,12 +444,12 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
 
     @Override
     public Filter getFilter() {
-        return mVideoAdapter.getFilter();
+        return mAdapter.getFilter();
     }
 
     @Override
     public void restoreList() {
-        mVideoAdapter.restoreList();
+        mAdapter.restoreList();
     }
 
     @Override
@@ -456,7 +466,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
 
     @Override
     public boolean onPrepareActionMode(ActionMode mode, Menu menu) {
-        int count = mVideoAdapter.getSelectionCount();
+        int count = mAdapter.getSelectionCount();
         if (count == 0) {
             stopActionMode();
             return false;
@@ -469,7 +479,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
 
     @Override
     public boolean onActionItemClicked(ActionMode mode, MenuItem item) {
-        List<MediaWrapper> list = mVideoAdapter.getSelection();
+        List<MediaWrapper> list = mAdapter.getSelection();
         if (!list.isEmpty()) {
             switch (item.getItemId()) {
                 case R.id.action_video_play:
@@ -482,8 +492,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
                     showInfoDialog(list.get(0));
                     break;
     //            case R.id.action_video_delete:
-    //                for (int position : mVideoAdapter.getSelectedPositions())
-    //                    removeVideo(position, mVideoAdapter.getItem(position));
+    //                for (int position : mAdapter.getSelectedPositions())
+    //                    removeVideo(position, mAdapter.getItem(position));
     //                break;
                 case R.id.action_video_download_subtitles:
                     MediaUtils.getSubs(getActivity(), list);
@@ -505,13 +515,13 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     @Override
     public void onDestroyActionMode(ActionMode mode) {
         mActionMode = null;
-        ArrayList<MediaWrapper> items = mVideoAdapter.getAll();
+        ArrayList<MediaWrapper> items = mAdapter.getAll();
         for (int i = 0; i < items.size(); ++i) {
             MediaWrapper mw = items.get(i);
             if (mw.hasStateFlags(MediaLibraryItem.FLAG_SELECTED)) {
                 mw.removeStateFlags(MediaLibraryItem.FLAG_SELECTED);
-                mVideoAdapter.resetSelectionCount();
-                mVideoAdapter.notifyItemChanged(i, VideoListAdapter.UPDATE_SELECTION);
+                mAdapter.resetSelectionCount();
+                mAdapter.notifyItemChanged(i, VideoListAdapter.UPDATE_SELECTION);
             }
         }
     }
@@ -545,8 +555,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
         MediaWrapper media = (MediaWrapper) item;
             if (mActionMode != null) {
                 item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
-                mVideoAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
-                mVideoAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
+                mAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+                mAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
                 invalidateActionMode();
                 return;
             }
@@ -559,7 +569,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
                 SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
                 if (settings.getBoolean("force_play_all", false)) {
                     ArrayList<MediaWrapper> playList = new ArrayList<>();
-                    MediaUtils.openList(activity, playList, mVideoAdapter.getListWithPosition(playList, position));
+                    MediaUtils.openList(activity, playList, mAdapter.getListWithPosition(playList, position));
                 } else {
                     playVideo(media, false);
                 }
@@ -571,8 +581,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
             if (mActionMode != null)
                 return false;
             item.toggleStateFlag(MediaLibraryItem.FLAG_SELECTED);
-            mVideoAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
-            mVideoAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
+            mAdapter.updateSelectionCount(item.hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+            mAdapter.notifyItemChanged(position, VideoListAdapter.UPDATE_SELECTION);
             startActionMode();
             return true;
     }
@@ -593,7 +603,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements MediaUpda
     }
 
     public void updateSeenMediaMarker() {
-        mVideoAdapter.setSeenMediaMarkerVisible(PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext()).getBoolean("media_seen", true));
-        mVideoAdapter.notifyItemRangeChanged(0, mVideoAdapter.getItemCount()-1, VideoListAdapter.UPDATE_SEEN);
+        mAdapter.setSeenMediaMarkerVisible(PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext()).getBoolean("media_seen", true));
+        mAdapter.notifyItemRangeChanged(0, mAdapter.getItemCount()-1, VideoListAdapter.UPDATE_SEEN);
     }
 }
