@@ -21,7 +21,6 @@
 package org.videolan.vlc.gui.video;
 
 import android.content.Context;
-import android.content.SharedPreferences;
 import android.databinding.BindingAdapter;
 import android.databinding.DataBindingUtil;
 import android.databinding.ViewDataBinding;
@@ -45,31 +44,24 @@ import org.videolan.medialibrary.media.MediaLibraryItem;
 import org.videolan.medialibrary.media.MediaWrapper;
 import org.videolan.vlc.BR;
 import org.videolan.vlc.R;
+import org.videolan.vlc.SortableAdapter;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.helpers.AsyncImageLoader;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.interfaces.IEventsHandler;
 import org.videolan.vlc.media.MediaGroup;
 import org.videolan.vlc.util.MediaItemFilter;
+import org.videolan.vlc.util.MediaLibraryItemComparator;
 
-import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Collections;
-import java.util.Comparator;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.Locale;
 
-public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.ViewHolder> implements Filterable {
+public class VideoListAdapter extends SortableAdapter<MediaWrapper, VideoListAdapter.ViewHolder> implements Filterable {
 
     public final static String TAG = "VLC/VideoListAdapter";
 
-    public final static int SORT_BY_TITLE = 0;
-    public final static int SORT_BY_LENGTH = 1;
-
-    public final static int SORT_BY_DATE = 2;
-
     final static int UPDATE_SELECTION = 0;
     final static int UPDATE_THUMB = 1;
     final static int UPDATE_TIME = 2;
@@ -77,9 +69,6 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     private boolean mListMode = false;
     private IEventsHandler mEventsHandler;
-    private VideoComparator mVideoComparator = new VideoComparator();
-    private ArrayList<MediaWrapper> mVideos = new ArrayList<>();
-    private ArrayDeque<ArrayList<MediaWrapper>> mPendingUpdates = new ArrayDeque<>();
     private ArrayList<MediaWrapper> mOriginalData = null;
     private ItemFilter mFilter = new ItemFilter();
     private int mSelectionCount = 0;
@@ -93,10 +82,6 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         mIsSeenMediaMarkerVisible = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext()).getBoolean("media_seen", true);
     }
 
-    VideoComparator getComparator() {
-        return mVideoComparator;
-    }
-
     @Override
     public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
         LayoutInflater inflater = (LayoutInflater) parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
@@ -112,7 +97,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     @Override
     public void onBindViewHolder(ViewHolder holder, int position) {
-        MediaWrapper media = mVideos.get(position);
+        MediaWrapper media = mDataset.get(position);
         if (media == null)
             return;
         holder.binding.setVariable(BR.scaleType, ImageView.ScaleType.CENTER_CROP);
@@ -128,7 +113,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         if (payloads.isEmpty())
             onBindViewHolder(holder, position);
         else {
-            MediaWrapper media = mVideos.get(position);
+            MediaWrapper media = mDataset.get(position);
             for (Object data : payloads) {
                 switch ((int) data) {
                     case UPDATE_THUMB:
@@ -159,50 +144,45 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     @Nullable
     public MediaWrapper getItem(int position) {
-        return isPositionValid(position) ? mVideos.get(position) : null;
+        return isPositionValid(position) ? mDataset.get(position) : null;
     }
 
     private boolean isPositionValid(int position) {
-        return position >= 0 && position < mVideos.size();
+        return position >= 0 && position < mDataset.size();
     }
 
     @MainThread
     public void add(MediaWrapper item) {
         ArrayList<MediaWrapper> list = new ArrayList<>(peekLast());
         list.add(item);
-        update(list, false);
+        update(list);
     }
 
     @MainThread
     public void remove(MediaWrapper item) {
         ArrayList<MediaWrapper> refList = new ArrayList<>(peekLast());
         if (refList.remove(item))
-            update(refList, false);
+            update(refList);
     }
 
     @MainThread
     public void addAll(Collection<MediaWrapper> items) {
-        mVideos.addAll(items);
+        mDataset.addAll(items);
         mOriginalData = null;
     }
 
-    @MainThread
-    private ArrayList<MediaWrapper> peekLast() {
-        return mPendingUpdates.isEmpty() ? mVideos : mPendingUpdates.peekLast();
-    }
-
     public boolean contains(MediaWrapper mw) {
-        return mVideos.indexOf(mw) != -1;
+        return mDataset.indexOf(mw) != -1;
     }
 
     public ArrayList<MediaWrapper> getAll() {
-        return mVideos;
+        return mDataset;
     }
 
     List<MediaWrapper> getSelection() {
         List<MediaWrapper> selection = new LinkedList<>();
-        for (int i = 0; i < mVideos.size(); ++i) {
-            MediaWrapper mw = mVideos.get(i);
+        for (int i = 0; i < mDataset.size(); ++i) {
+            MediaWrapper mw = mDataset.get(i);
             if (mw.hasStateFlags(MediaLibraryItem.FLAG_SELECTED)) {
                 if (mw instanceof MediaGroup)
                     selection.addAll(((MediaGroup) mw).getAll());
@@ -231,10 +211,10 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     @MainThread
     public void update(MediaWrapper item) {
-        int position = mVideos.indexOf(item);
+        int position = mDataset.indexOf(item);
         if (position != -1) {
-            if (!(mVideos.get(position) instanceof MediaGroup))
-                mVideos.set(position, item);
+            if (!(mDataset.get(position) instanceof MediaGroup))
+                mDataset.set(position, item);
             notifyItemChanged(position, UPDATE_THUMB);
         } else
             add(item);
@@ -242,7 +222,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     @MainThread
     public void clear() {
-        mVideos.clear();
+        mDataset.clear();
         mOriginalData = null;
     }
 
@@ -299,7 +279,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     @Override
     public int getItemCount() {
-        return mVideos.size();
+        return mDataset.size();
     }
 
     @Override
@@ -311,7 +291,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         MediaWrapper mw;
         int offset = 0;
         for (int i = 0; i < getItemCount(); ++i) {
-            mw = mVideos.get(i);
+            mw = mDataset.get(i);
             if (mw instanceof MediaGroup) {
                 for (MediaWrapper item : ((MediaGroup) mw).getAll())
                     list.add(item);
@@ -338,7 +318,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
         public void onClick(View v) {
             int position = getLayoutPosition();
-            mEventsHandler.onClick(v, position, mVideos.get(position));
+            mEventsHandler.onClick(v, position, mDataset.get(position));
         }
 
         public void onMoreClick(View v){
@@ -347,7 +327,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
         public boolean onLongClick(View v) {
             int position = getLayoutPosition();
-            return mEventsHandler.onLongClick(v, position, mVideos.get(position));
+            return mEventsHandler.onLongClick(v, position, mDataset.get(position));
         }
 
         private void setOverlay(boolean selected) {
@@ -356,105 +336,13 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
         @Override
         public void onFocusChange(View v, boolean hasFocus) {
-            setViewBackground(hasFocus || mVideos.get(getLayoutPosition()).hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
+            setViewBackground(hasFocus || mDataset.get(getLayoutPosition()).hasStateFlags(MediaLibraryItem.FLAG_SELECTED));
         }
 
         private void setViewBackground(boolean highlight) {
             itemView.setBackgroundColor(highlight ? UiTools.ITEM_FOCUS_ON : UiTools.ITEM_FOCUS_OFF);
         }
     }
-    int sortDirection(int sortDirection) {
-        return mVideoComparator.sortDirection(sortDirection);
-    }
-
-    void sortBy(int sortby) {
-        mVideoComparator.sortBy(sortby);
-    }
-
-    private class VideoComparator implements Comparator<MediaWrapper> {
-
-        private static final String KEY_SORT_BY =  "sort_by";
-        private static final String KEY_SORT_DIRECTION =  "sort_direction";
-
-        private int mSortDirection;
-        private int mSortBy;
-        protected SharedPreferences mSettings = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
-
-        VideoComparator() {
-            mSortBy = mSettings.getInt(KEY_SORT_BY, SORT_BY_TITLE);
-            mSortDirection = mSettings.getInt(KEY_SORT_DIRECTION, 1);
-        }
-
-        int sortDirection(int sortby) {
-            if (sortby == mSortBy)
-                return  mSortDirection;
-            else
-                return -1;
-        }
-
-        void sortBy(int sortby) {
-            switch (sortby) {
-                case SORT_BY_TITLE:
-                    if (mSortBy == SORT_BY_TITLE)
-                        mSortDirection *= -1;
-                    else {
-                        mSortBy = SORT_BY_TITLE;
-                        mSortDirection = 1;
-                    }
-                    break;
-                case SORT_BY_LENGTH:
-                    if (mSortBy == SORT_BY_LENGTH)
-                        mSortDirection *= -1;
-                    else {
-                        mSortBy = SORT_BY_LENGTH;
-                        mSortDirection *= 1;
-                    }
-                    break;
-                case SORT_BY_DATE:
-                    if (mSortBy == SORT_BY_DATE)
-                        mSortDirection *= -1;
-                    else {
-                        mSortBy = SORT_BY_DATE;
-                        mSortDirection *= 1;
-                    }
-                    break;
-                default:
-                    mSortBy = SORT_BY_TITLE;
-                    mSortDirection = 1;
-                    break;
-            }
-            ArrayList<MediaWrapper> list = new ArrayList<>(mVideos);
-            update(list, true);
-
-            mSettings.edit()
-                    .putInt(KEY_SORT_BY, mSortBy)
-                    .putInt(KEY_SORT_DIRECTION, mSortDirection)
-                    .apply();
-        }
-
-        @Override
-        public int compare(MediaWrapper item1, MediaWrapper item2) {
-            if (item1 == null)
-                return item2 == null ? 0 : -1;
-            else if (item2 == null)
-                return 1;
-
-            int compare = 0;
-            switch (mSortBy) {
-                case SORT_BY_TITLE:
-                    compare = item1.getTitle().toUpperCase(Locale.ENGLISH).compareTo(
-                            item2.getTitle().toUpperCase(Locale.ENGLISH));
-                    break;
-                case SORT_BY_LENGTH:
-                    compare = ((Long) item1.getLength()).compareTo(item2.getLength());
-                    break;
-                case SORT_BY_DATE:
-                    compare = ((Long) item1.getLastModified()).compareTo(item2.getLastModified());
-                    break;
-            }
-            return mSortDirection * compare;
-        }
-    }
 
     @Override
     public Filter getFilter() {
@@ -464,7 +352,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
     @MainThread
     void restoreList() {
         if (mOriginalData != null) {
-            update(new ArrayList<>(mOriginalData), false);
+            update(new ArrayList<>(mOriginalData));
             mOriginalData = null;
         }
     }
@@ -474,9 +362,9 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         @Override
         protected List<MediaWrapper> initData() {
             if (mOriginalData == null) {
-                mOriginalData = new ArrayList<>(mVideos.size());
-                for (int i = 0; i < mVideos.size(); ++i)
-                    mOriginalData.add(mVideos.get(i));
+                mOriginalData = new ArrayList<>(mDataset.size());
+                for (int i = 0; i < mDataset.size(); ++i)
+                    mOriginalData.add(mDataset.get(i));
             }
             return mOriginalData;
         }
@@ -484,7 +372,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         @Override
         protected void publishResults(CharSequence charSequence, FilterResults filterResults) {
             //noinspection unchecked
-            update((ArrayList<MediaWrapper>) filterResults.values, false);
+            update((ArrayList<MediaWrapper>) filterResults.values);
         }
     }
 
@@ -497,32 +385,15 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         view.setLayoutParams(layoutParams);
     }
 
-    @MainThread
-    void update(final ArrayList<MediaWrapper> items, final boolean detectMoves) {
-        mPendingUpdates.add(items);
-        if (mPendingUpdates.size() == 1)
-            internalUpdate(items, detectMoves);
+    @Override
+    protected DiffUtil.Callback createCB(final ArrayList<MediaWrapper> items) {
+        return new VideoItemDiffCallback(mDataset, items);
     }
 
-    private void internalUpdate(final ArrayList<MediaWrapper> items, final boolean detectMoves) {
-        VLCApplication.runBackground(new Runnable() {
-            @Override
-            public void run() {
-                Collections.sort(items, mVideoComparator);
-                final DiffUtil.DiffResult result = DiffUtil.calculateDiff(new VideoItemDiffCallback(mVideos, items), detectMoves);
-                VLCApplication.runOnMainThread(new Runnable() {
-                    @Override
-                    public void run() {
-                        mPendingUpdates.remove();
-                        mVideos = items;
-                        result.dispatchUpdatesTo(VideoListAdapter.this);
-                        mEventsHandler.onUpdateFinished(null);
-                        if (!mPendingUpdates.isEmpty())
-                            internalUpdate(mPendingUpdates.peek(), true);
-                    }
-                });
-            }
-        });
+    @Override
+    protected void onUpdateFinished() {
+        super.onUpdateFinished();
+        mEventsHandler.onUpdateFinished(null);
     }
 
     private class VideoItemDiffCallback extends DiffUtil.Callback {
@@ -546,16 +417,17 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         public boolean areItemsTheSame(int oldItemPosition, int newItemPosition) {
             MediaWrapper oldItem = oldList.get(oldItemPosition);
             MediaWrapper newItem = newList.get(newItemPosition);
-            return oldList != null && newList != null && oldItem.getType() == newItem.getType() && oldItem.equals(newItem);
+            return oldItem == newItem || (oldList != null && newList != null
+                    && oldItem.getType() == newItem.getType() && oldItem.equals(newItem));
         }
 
         @Override
         public boolean areContentsTheSame(int oldItemPosition, int newItemPosition) {
             MediaWrapper oldItem = oldList.get(oldItemPosition);
             MediaWrapper newItem = newList.get(newItemPosition);
-            return oldItem.getTime() == newItem.getTime()
+            return oldItem == newItem || (oldItem.getTime() == newItem.getTime()
                     && TextUtils.equals(oldItem.getArtworkMrl(), newItem.getArtworkMrl())
-                    && oldItem.getSeen() == newItem.getSeen();
+                    && oldItem.getSeen() == newItem.getSeen());
         }
 
         @Nullable
@@ -575,4 +447,16 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
     public void setSeenMediaMarkerVisible(boolean seenMediaMarkerVisible) {
         mIsSeenMediaMarkerVisible = seenMediaMarkerVisible;
     }
+
+    @Override
+    protected boolean isSortAllowed(int sort) {
+        switch (sort) {
+            case MediaLibraryItemComparator.SORT_BY_TITLE:
+            case MediaLibraryItemComparator.SORT_BY_DATE:
+            case MediaLibraryItemComparator.SORT_BY_LENGTH:
+                return true;
+            default:
+                return false;
+        }
+    }
 }
